@@ -1,11 +1,13 @@
+from collections import defaultdict
 from contextlib import contextmanager
 from textwrap import indent
 from typing import Optional
 from lark import Tree
 from lark.visitors import Interpreter
-from gersemi.ast_helpers import contains_line_comment
+from gersemi.ast_helpers import is_line_comment_in
 from gersemi.configuration import Indent, ListExpansion, Tabs
 from gersemi.types import Nodes
+from gersemi.warnings import FormatterWarnings, UnknownCommandWarning
 
 
 def get_indent(indent_type: Indent) -> str:
@@ -14,6 +16,10 @@ def get_indent(indent_type: Indent) -> str:
     return " " * indent_type
 
 
+class WontFit(Exception):
+    pass
+
+
 class BaseDumper(Interpreter):
     def __init__(self, width, indent_type):
         self.width = width
@@ -21,6 +27,7 @@ class BaseDumper(Interpreter):
         self._indent_symbol = get_indent(self.indent_type)
         self.indent_level = 0
         self.favour_expansion = False
+        self.unknown_commands_used = defaultdict(list)
 
     def __default__(self, tree: Tree):
         return "".join(self.visit_children(tree))
@@ -32,21 +39,40 @@ class BaseDumper(Interpreter):
     def _indent(self, text: str):
         return indent(text, self.indent_symbol)
 
+    def _single_line_helper(self, children, offset):
+        width = offset
+        for c in children:
+            if isinstance(c, Tree):
+                if is_line_comment_in(c):
+                    raise WontFit()
+
+                result = self.visit(c)
+            else:
+                result = c
+
+            if "\n" in result:
+                raise WontFit()
+
+            width += len(result)
+            if width > self.width:
+                raise WontFit()
+
+            yield result
+            width += 1
+
     def _try_to_format_into_single_line(
-        self, children: Nodes, separator: str = "", prefix: str = "", postfix: str = ""
+        self, children: Nodes, prefix: str = "", postfix: str = ""
     ) -> Optional[str]:
         if self.favour_expansion:
             return None
 
-        if not contains_line_comment(children):
+        try:
+            reserved_space = len(prefix) + len(postfix) + len(self.indent_symbol)
             with self.not_indented():
-                formatted_children = separator.join(
-                    self.visit(c) if isinstance(c, Tree) else c for c in children
-                )
-            result = self._indent(f"{prefix}{formatted_children}{postfix}")
-            if len(result) <= self.width and "\n" not in result:
-                return result
-        return None
+                formatted = " ".join(self._single_line_helper(children, reserved_space))
+            return f"{self.indent_symbol}{prefix}{formatted}{postfix}"
+        except WontFit:
+            return None
 
     @contextmanager
     def with_indent_level(self, indent_level):
@@ -83,3 +109,15 @@ class BaseDumper(Interpreter):
 
     def format_command_name(self, identifier):
         return identifier.lower()
+
+    def _record_unknown_command(self, command):
+        self.unknown_commands_used[str(command)].append((command.line, command.column))
+
+    def get_warnings(self) -> FormatterWarnings:
+        if len(self.unknown_commands_used) == 0:
+            return []
+
+        return [
+            UnknownCommandWarning(command_name=name, positions=positions)
+            for name, positions in self.unknown_commands_used.items()
+        ]
