@@ -16,6 +16,9 @@ class DropWhitespaces(Transformer):
             return Discard
         return Tree("non_command_element", children)
 
+    def arguments(self, children):
+        return Tree("arguments", set(children))
+
     NEWLINE = _drop_node
     newline_or_gap = _drop_node
 
@@ -30,6 +33,6 @@ def check_abstract_syntax_trees_equivalence(lhs, rhs):
         raise ASTMismatch
 
 
-def check_code_equivalence(parser, lhs, rhs):
-    lhs_parsed, rhs_parsed = parser.parse(lhs), parser.parse(rhs)
+def check_code_equivalence(parser, lhs_parsed, rhs):
+    rhs_parsed = parser.parse(rhs)
     check_abstract_syntax_trees_equivalence(lhs_parsed, rhs_parsed)
